@@ -96,6 +96,7 @@ IconView::IconView(QGraphicsWidget *parent)
 
     m_scrollBar->hide();
     connect(m_scrollBar, SIGNAL(valueChanged(int)), SLOT(repositionWidgetsManually()));
+    connect(m_scrollBar, SIGNAL(valueChanged(int)), SLOT(viewScrolled()));
 
     m_toolTipWidget = new ToolTipWidget(this);
     m_toolTipWidget->hide();
@@ -145,6 +146,7 @@ void IconView::setIconSize(const QSize &size)
     if (size != m_iconSize) {
         m_iconSize = size;
         updateGridSize();
+        updateActionButtons();
     }
 }
 
@@ -261,9 +263,10 @@ bool IconView::customLayout() const
 
 void IconView::setIconPositionsData(const QStringList &data)
 {
-    // Sanity checks
-    if (data.size() < 5 || data.at(0).toInt() != 1 || ((data.size() - 2) % 3) ||
-        data.at(1).toInt() != ((data.size() - 2) / 3)) {
+    if (data.size() < 5 ||                               // is there data stored for at least 1 icon?
+        data.at(0).toInt() != 1 ||                       // is format version number 1?
+        ((data.size() - 2) % 3) ||                       // are there 3 strings stored for every icon?
+        data.at(1).toInt() != ((data.size() - 2) / 3)) { // is the specified number of icons equal to the stored number of icon entries?
         return;
     }
 
@@ -934,7 +937,7 @@ void IconView::finishedScrolling()
 {
     // Find the bounding rect of the items
     QRect boundingRect = itemsBoundingRect();
-    
+
     if (boundingRect.isValid()) {
         // Add the margin
         boundingRect.adjust(-10, -10, 10, 10);
@@ -1557,6 +1560,49 @@ void IconView::updateToolTip()
     }
 }
 
+void IconView::updateRubberband()
+{
+    const int scrollBarOffset = m_scrollBar->isVisible() ? m_scrollBar->geometry().width() : 0;
+    const QRect realContentsRect = itemsBoundingRect().adjusted(-10, -10, 10, 10) | contentsRect().toAlignedRect();
+    const QRect viewportRect = layoutDirection() == Qt::LeftToRight ?
+                   realContentsRect.adjusted(0, 0, int(-scrollBarOffset), 0) :
+                   realContentsRect.adjusted(int(scrollBarOffset), 0, 0, 0);
+    const QPointF pos = mapToViewport(m_mouseMovedPos);
+    const QRectF rubberBand = QRectF(m_buttonDownPos, pos).normalized();
+    const QRect r = QRectF(rubberBand & viewportRect).toAlignedRect();
+
+    const QModelIndex prevHoveredIndex = m_hoveredIndex;
+
+    if (r != m_rubberBand) {
+        const QPoint pt = pos.toPoint();
+        QRectF dirtyRect = m_rubberBand | r;
+        m_rubberBand = r;
+
+        dirtyRect |= visualRect(m_hoveredIndex);
+        m_hoveredIndex = QModelIndex();
+
+        repaintSelectedIcons();
+        selectIconsInArea(m_rubberBand, pt);
+
+        markAreaDirty(dirtyRect);
+    }
+
+    if (prevHoveredIndex != m_hoveredIndex) {
+        if (prevHoveredIndex.isValid()) {
+            emit left(prevHoveredIndex);
+        }
+        if (m_hoveredIndex.isValid()) {
+            emit entered(m_hoveredIndex);
+        }
+    }
+}
+
+void IconView::updateActionButtons()
+{
+    m_actionOverlay->setShowFolderButton(clickToViewFolders());
+    m_actionOverlay->setShowSelectionButton(showSelectionMarker());
+}
+
 void IconView::checkIfFolderResult(const QModelIndex &index, bool isFolder)
 {
     m_toolTipShowTimer.stop();
@@ -1587,6 +1633,14 @@ void IconView::svgChanged()
 
     // this updates the grid size, then calls layoutItems() which in turn repaints the view
     updateGridSize();
+    updateActionButtons();
+}
+
+void IconView::viewScrolled()
+{
+    if (m_rubberBand.isValid()) {
+        updateRubberband();
+    }
 }
 
 void IconView::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
@@ -1596,7 +1650,7 @@ void IconView::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
         emit entered(index);
         m_hoveredIndex = index;
         markAreaDirty(visualRect(index));
-        if (!m_clickToViewFolders) {
+        if (!clickToViewFolders()) {
             AsyncFileTester::checkIfFolder(m_hoveredIndex, this, "checkIfFolderResult");
         }
     }
@@ -1632,7 +1686,7 @@ void IconView::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
         m_hoveredIndex = index;
         updateToolTip();
 
-        if (!m_clickToViewFolders) {
+        if (!clickToViewFolders()) {
             AsyncFileTester::checkIfFolder(m_hoveredIndex, this, "checkIfFolderResult");
         }
     }
@@ -1803,13 +1857,19 @@ void IconView::mousePressEvent(QGraphicsSceneMouseEvent *event)
         const QModelIndex index = indexAt(pos);
         if (index.isValid()) {
             if (!m_selectionModel->isSelected(index)) {
+                // A previously unselected icon clicked, clear and repaint selection
+                // then select and repaint the icon at index
+                const QRect dirtyRect = selectedItemsBoundingRect();
                 m_selectionModel->select(index, QItemSelectionModel::ClearAndSelect);
                 m_selectionModel->setCurrentIndex(index, QItemSelectionModel::NoUpdate);
-                markAreaDirty(visibleArea());
+                markAreaDirty(dirtyRect);
+                markAreaDirty(visualRect(index));
             }
         } else if (m_selectionModel->hasSelection()) {
+            // Empty space clicked, clear and repaint selection
+            const QRect dirtyRect = selectedItemsBoundingRect();
             m_selectionModel->clearSelection();
-            markAreaDirty(visibleArea());
+            markAreaDirty(dirtyRect);
         }
         event->ignore(); // Causes contextMenuEvent() to get called
         return;
@@ -1820,7 +1880,7 @@ void IconView::mousePressEvent(QGraphicsSceneMouseEvent *event)
 
         // If an icon was pressed
         if (index.isValid()) {
-            //if ctrl is held
+            // If ctrl is held
             if (event->modifiers() & Qt::ControlModifier) {
                 m_selectionModel->select(index, QItemSelectionModel::Toggle);
                 m_selectionModel->setCurrentIndex(index, QItemSelectionModel::NoUpdate);
@@ -1833,20 +1893,20 @@ void IconView::mousePressEvent(QGraphicsSceneMouseEvent *event)
                 else {
                     selectIconRange(current, index);
                 }
-            } else if (!m_selectionModel->isSelected(index)) {
-                //if not already selected
-                m_selectionModel->select(index, QItemSelectionModel::ClearAndSelect);
-                m_selectionModel->setCurrentIndex(index, QItemSelectionModel::NoUpdate);
-                markAreaDirty(visibleArea());
             } else {
+                if (!m_selectionModel->isSelected(index)) {
+                    // If not already selected
+                    const QRect dirtyRect = selectedItemsBoundingRect();
+                    m_selectionModel->select(index, QItemSelectionModel::ClearAndSelect);
+                    m_selectionModel->setCurrentIndex(index, QItemSelectionModel::NoUpdate);
+                    markAreaDirty(dirtyRect);
+                }
+                m_drawIconShrinked = KGlobalSettings::singleClick();
                 markAreaDirty(visualRect(index));
             }
 
             m_pressedIndex = index;
             m_buttonDownPos = pos;
-            if (KGlobalSettings::singleClick()) {
-              m_drawIconShrinked = true;
-            }
             return;
         }
 
@@ -1872,8 +1932,9 @@ void IconView::mousePressEvent(QGraphicsSceneMouseEvent *event)
 
         if (m_selectionModel->hasSelection()) {
             if (!(event->modifiers() & (Qt::ShiftModifier | Qt::ControlModifier))) {
+                const QRect dirtyRect = selectedItemsBoundingRect();
                 m_selectionModel->clearSelection();
-                markAreaDirty(visibleArea());
+                markAreaDirty(dirtyRect);
             }
         }
     }
@@ -1889,6 +1950,7 @@ void IconView::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
         if (m_rubberBand.isValid()) {
             markAreaDirty(m_rubberBand);
             m_rubberBand = QRect();
+            stopAutoScrolling();
             return;
         }
 
@@ -1904,12 +1966,11 @@ void IconView::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
                     // we ignore double clicks when in single click mode
                     if (!m_doubleClick) {
                         emit activated(index);
-                        m_selectionModel->clearSelection();
-                        markAreaDirty(visibleArea());
+                        markAreaDirty(visualRect(index));
                     }
                 } else {
                     // since icon shrinking is delayed, we always repaint after mouse release
-                    markAreaDirty(visibleArea());
+                    markAreaDirty(visualRect(index));
                 }
 
                 // We don't clear and update the selection and current index in
@@ -1917,9 +1978,10 @@ void IconView::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
                 // so we need to do that here.
                 if (m_selectionModel->currentIndex() != index ||
                     m_selectionModel->selectedIndexes().count() > 1) {
+                    const QRect dirtyRect = selectedItemsBoundingRect();
                     m_selectionModel->select(index, QItemSelectionModel::ClearAndSelect);
                     m_selectionModel->setCurrentIndex(index, QItemSelectionModel::NoUpdate);
-                    markAreaDirty(visibleArea());
+                    markAreaDirty(dirtyRect);
                 }
             }
         }
@@ -1956,8 +2018,10 @@ void IconView::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
     // Activate the item
     emit activated(index);
 
-    m_selectionModel->clearSelection();
-    markAreaDirty(visibleArea());
+    // A double-click implies a single-click, which means that the selection will
+    // be reset and repainted on mousePressEvent
+    // so we only need to repant the icon itself to get the "sunken" effect
+    markAreaDirty(visualRect(index));
 }
 
 void IconView::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
@@ -1976,38 +2040,21 @@ void IconView::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
         return;
     }
 
-    const int scrollBarOffset = m_scrollBar->isVisible() ? m_scrollBar->geometry().width() : 0;
-    const QRect viewportRect = layoutDirection() == Qt::LeftToRight ?
-                   visibleArea().adjusted(0, 0, int(-scrollBarOffset), 0) :
-                   visibleArea().adjusted(int(scrollBarOffset), 0, 0, 0);
-    const QPointF pos = mapToViewport(event->pos());
-    const QRectF rubberBand = QRectF(m_buttonDownPos, pos).normalized();
-    const QRect r = QRectF(rubberBand & viewportRect).toAlignedRect();
+    m_mouseMovedPos = event->pos();
 
-    const QModelIndex prevHoveredIndex = m_hoveredIndex;
-
-    if (r != m_rubberBand) {
-        const QPoint pt = pos.toPoint();
-        QRectF dirtyRect = m_rubberBand | r;
-        m_rubberBand = r;
-
-        dirtyRect |= visualRect(m_hoveredIndex);
-        m_hoveredIndex = QModelIndex();
-
-        repaintSelectedIcons();
-        selectIconsInArea(m_rubberBand, pt);
-
-        markAreaDirty(dirtyRect);
+    const int maxSpeed = 500; // Pixels per second
+    const int distance = gridSize().height() * .75;
+    if (event->pos().y() < contentsRect().y() + distance) {
+        int speed = maxSpeed / distance * (distance - event->pos().y() - contentsRect().y());
+        autoScroll(ScrollUp, speed);
+    } else if (event->pos().y() > contentsRect().bottom() - distance) {
+        int speed = maxSpeed / distance * (event->pos().y() - contentsRect().bottom() + distance);
+        autoScroll(ScrollDown, speed);
+    } else {
+        stopAutoScrolling();
     }
 
-    if (prevHoveredIndex != m_hoveredIndex) {
-        if (prevHoveredIndex.isValid()) {
-            emit left(prevHoveredIndex);
-        }
-        if (m_hoveredIndex.isValid()) {
-            emit entered(m_hoveredIndex);
-        }
-    }
+    updateRubberband();
 }
 
 void IconView::wheelEvent(QGraphicsSceneWheelEvent *event)
@@ -2021,6 +2068,7 @@ void IconView::wheelEvent(QGraphicsSceneWheelEvent *event)
         return;
     }
 
+    stopAutoScrolling();
     int pixels = 64 * event->delta() / 120;
     smoothScroll(0, -pixels);
 }
@@ -2595,13 +2643,18 @@ void IconView::selectIconRange(const QModelIndex &begin, const QModelIndex &end)
     repaintSelectedIcons();
 }
 
+QRect IconView::selectedItemsBoundingRect() const
+{
+    QRect rect;
+    foreach (const QModelIndex &index, m_selectionModel->selectedIndexes()) {
+            rect |= visualRect(index);
+    }
+    return rect;
+}
+
 void IconView::repaintSelectedIcons()
 {
-    QRect dirtyRect;
-    foreach (const QModelIndex &index, m_selectionModel->selectedIndexes()) {
-            dirtyRect |= visualRect(index);
-    }
-    markAreaDirty(dirtyRect);
+    markAreaDirty(selectedItemsBoundingRect());
 }
 
 void IconView::popupCloseRequested()
@@ -2615,12 +2668,12 @@ void IconView::popupCloseRequested()
 void IconView::setClickToViewFolders(bool click)
 {
     m_clickToViewFolders = click;
-    m_actionOverlay->setShowFolderButton(click);
+    m_actionOverlay->setShowFolderButton(clickToViewFolders());
 }
 
 bool IconView::clickToViewFolders() const
 {
-    return m_clickToViewFolders;
+    return overlayEnabled() ? m_clickToViewFolders : false;
 }
 
 void IconView::openPopup(const QModelIndex &index)
@@ -2674,12 +2727,18 @@ void IconView::openPopup(const QModelIndex &index)
 void IconView::setShowSelectionMarker(bool show)
 {
     m_showSelectionMarker = show;
-    m_actionOverlay->setShowSelectionButton(show);
+    m_actionOverlay->setShowSelectionButton(showSelectionMarker());
 }
 
 bool IconView::showSelectionMarker() const
 {
-    return m_showSelectionMarker;
+    return overlayEnabled() ? m_showSelectionMarker : false;
+}
+
+bool IconView::overlayEnabled() const
+{
+    // Do not let the action overlay cover the icon
+    return gridSize().width() - m_iconSize.width() > 2*qMin(m_actionOverlay->iconSize().width(), m_actionOverlay->iconSize().height());
 }
 
 void IconView::timerEvent(QTimerEvent *event)
